SystemOrganization addCategory: #'ToolBuilder-MVC'!

ToolBuilderTests subclass: #MVCToolBuilderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-MVC'!

!MVCToolBuilderTests commentStamp: 'ar 2/11/2005 15:02' prior: 0!
Tests for the MVC tool builder.!

!MVCToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 19:28'!
acceptWidgetText
	widget hasUnacceptedEdits: true.
	widget controller accept.! !

!MVCToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 19:22'!
changeListWidget
	widget changeModelSelection: widget getCurrentSelectionIndex + 1.! !

!MVCToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 19:16'!
fireButtonWidget
	widget performAction.! !

!MVCToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 14:46'!
setUp
	super setUp.
	builder := MVCToolBuilder new.! !

ToolBuilder subclass: #MVCToolBuilder
	instanceVariableNames: 'panes topSize widgets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-MVC'!

!MVCToolBuilder commentStamp: 'ar 2/11/2005 15:02' prior: 0!
The MVC tool builder.!

!MVCToolBuilder class methodsFor: 'accessing' stamp: 'ar 2/12/2005 18:49'!
isActiveBuilder
	"Answer whether I am the currently active builder"
	"This is really a way of answering whether 'Smalltalk isMVC'"
	ScheduledControllers ifNil:[^false].
	^ScheduledControllers activeControllerProcess == Processor activeProcess! !

!MVCToolBuilder methodsFor: 'private' stamp: 'ar 2/10/2005 00:18'!
asWindow: aRectangle
	^(aRectangle origin * topSize extent) truncated
		corner: (aRectangle corner * topSize extent) truncated! !

!MVCToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 7/14/2005 22:29'!
buildPluggableButton: aSpec
	| widget label state |
	label := aSpec label.
	state := aSpec state.
	widget := PluggableButtonView on: aSpec model
				getState: (state isSymbol ifTrue:[state])
				action: aSpec action
				label: (label isSymbol ifTrue:[label]).
	self register: widget id: aSpec name.
	(label isSymbol or:[label == nil]) ifFalse:[widget label: label].
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addSubView: widget].
	^widget! !

!MVCToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 7/14/2005 22:29'!
buildPluggableList: aSpec
	| widget listClass getIndex setIndex |
	aSpec getSelected ifNil:[
		listClass := PluggableListView.
		getIndex := aSpec getIndex.
		setIndex := aSpec setIndex.
	] ifNotNil:[
		listClass := PluggableListViewByItem.
		getIndex := aSpec getSelected.
		setIndex := aSpec setSelected.
	].
	widget := listClass on: aSpec model
				list: aSpec list
				selected: getIndex
				changeSelected: setIndex
				menu: aSpec menu
				keystroke: aSpec keyPress.
	self register: widget id: aSpec name.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addSubView: widget].
	panes ifNotNil:[
		aSpec list ifNotNil:[panes add: aSpec list].
	].
	^widget! !

!MVCToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 7/14/2005 22:29'!
buildPluggablePanel: aSpec
	| widget children |
	widget := View new model: aSpec model.
	self register: widget id: aSpec name.
	children := aSpec children.
	children isSymbol ifTrue:[
		"@@@@ FIXME: PluggablePanes need to remember their getChildrenSelector"
		"widget getChildrenSelector: children.
		widget update: children."
		children := #().
	].
	self buildAll: children in: widget.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addSubView: widget].
	self setLayout: aSpec layout in: widget.
	^widget! !

!MVCToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 7/14/2005 22:29'!
buildPluggableText: aSpec
	| widget |
	widget := PluggableTextView on: aSpec model
				text: aSpec getText 
				accept: aSpec setText
				readSelection: aSpec selection 
				menu: aSpec menu.
	self register: widget id: aSpec name.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addSubView: widget].
	panes ifNotNil:[
		aSpec getText ifNotNil:[panes add: aSpec getText].
	].
	^widget! !

!MVCToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 7/14/2005 22:30'!
buildPluggableWindow: aSpec
	| widget children |
	topSize := 0@0 corner: 640@480.
	aSpec layout == #proportional ifFalse:[
		"This needs to be implemented - probably by adding a single pane and then the rest"
		^self error: 'Not implemented'.
	].
	widget := StandardSystemView new.
	self register: widget id: aSpec name.
	widget model: aSpec model.
	children := aSpec children.
	children isSymbol ifTrue:[
		"This isn't implemented by StandardSystemView, so we fake it"
		children := widget model perform: children.
	].
	aSpec extent ifNotNil:[topSize :=  0@0 extent: aSpec extent].
	widget window: topSize.
	panes := OrderedCollection new.
	self buildAll: children in: widget.
	widget setUpdatablePanesFrom: panes.
	^widget! !

!MVCToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:42'!
close: aWidget
	"Close a previously opened widget"
	aWidget controller closeAndUnschedule.! !

!MVCToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:41'!
open: anObject
	"Build and open the object. Answer the widget opened."
	| window |
	window := self build: anObject.
	window controller open.
	^window! !

!MVCToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:42'!
open: anObject label: aString
	"Build an open the object, labeling it appropriately.  Answer the widget opened."
	| window |
	window := self build: anObject.
	window label: aString.
	window controller open.
	^window! !

!MVCToolBuilder methodsFor: 'defaults' stamp: 'ar 2/12/2005 17:42'!
pluggableTreeSpec
	"We have no tree widget in MVC right now"
	^nil! !

!MVCToolBuilder methodsFor: 'private' stamp: 'ar 7/14/2005 22:30'!
register: widget id: id
	id ifNil:[^self].
	widgets ifNil:[widgets := Dictionary new].
	widgets at: id put: widget.! !

!MVCToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:46'!
runModal: aWidget
	"Run the (previously opened) widget modally, e.g., 
	do not return control to the sender before the user has responded."

	self error: 'Please implement me'.! !

!MVCToolBuilder methodsFor: 'private' stamp: 'ar 2/10/2005 00:17'!
setFrame: aRectangle in: widget
	| win |
	aRectangle ifNil:[^nil].
	win := self asWindow: aRectangle.
	widget window: win.! !

!MVCToolBuilder methodsFor: 'private' stamp: 'ar 2/10/2005 21:33'!
setLayout: layout in: widget
	layout == #proportional ifTrue:[^self].
	layout == #horizontal ifTrue:[
		| prev |
		prev := nil.
		widget subViews do:[:next|
			prev ifNotNil:[
				next align: next viewport topLeft with: prev viewport topRight.
			].
			prev := next.
		].
		^self].
	layout == #vertical ifTrue:[
		| prev |
		prev := nil.
		widget subViews do:[:next|
			prev ifNotNil:[
				next align: next viewport topLeft with: prev viewport bottomLeft.
			].
			prev := next.
		].
		^self].
	^self error: 'Unknown layout: ', layout.! !

!MVCToolBuilder methodsFor: 'private' stamp: 'ar 7/14/2005 22:30'!
widgetAt: id ifAbsent: aBlock
	widgets ifNil:[^aBlock value].
	^widgets at: id ifAbsent: aBlock! !

UIManager subclass: #MVCUIManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-MVC'!

!MVCUIManager commentStamp: 'ar 2/11/2005 21:53' prior: 0!
The MVC ui manager.!

!MVCUIManager class methodsFor: 'accessing' stamp: 'ar 2/13/2005 00:09'!
isActiveManager
	"Answer whether I should act as the active ui manager"
	"This is really a way of answering whether 'Smalltalk isMVC'"
	ScheduledControllers ifNil:[^false].
	^ScheduledControllers activeControllerProcess == Processor activeProcess! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/11/2005 15:43'!
chooseFrom: aList lines: linesArray title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	| menu |
	menu := PopUpMenu labelArray: aList lines: linesArray.
	^aString isEmpty ifTrue:[menu startUp] ifFalse:[menu startUpWithCaption: aString]! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/11/2005 15:43'!
confirm: queryString
	"Put up a yes/no menu with caption queryString. Answer true if the 
	response is yes, false if no. This is a modal question--the user must 
	respond yes or no."
	^PopUpMenu confirm: queryString! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/11/2005 15:43'!
confirm: aString orCancel: cancelBlock
	"Put up a yes/no/cancel menu with caption aString. Answer true if  
	the response is yes, false if no. If cancel is chosen, evaluate  
	cancelBlock. This is a modal question--the user must respond yes or no."
	^PopUpMenu confirm: aString orCancel: cancelBlock! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 6/25/2005 18:07'!
displayProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock
	"Display titleString as a caption over a progress bar while workBlock is evaluated."
	^ProgressInitiationException 
		display: titleString
		at: aPoint 
		from: minVal 
		to: maxVal 
		during: workBlock! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/28/2005 17:07'!
informUserDuring: aBlock
	"Display a message above (or below if insufficient room) the cursor 
	during execution of the given block.
		UIManager default informUserDuring:[:bar|
			#(one two three) do:[:info|
				bar value: info.
				(Delay forSeconds: 1) wait]]"
	aBlock value:[:string| Transcript cr; show: string].! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/28/2005 20:40'!
informUser: aString during: aBlock
	"Display a message above (or below if insufficient room) the cursor 
	during execution of the given block.
		UIManager default informUser: 'Just a sec!!' during: [(Delay forSeconds: 1) wait].
	"
	(SelectionMenu labels: '')
		displayAt: Sensor cursorPoint
		withCaption: aString
		during: [aBlock value]! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/11/2005 15:43'!
inform: aString
	"Display a message for the user to read and then dismiss"
	^PopUpMenu inform: aString! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/28/2005 17:05'!
multiLineRequest: queryString centerAt: aPoint initialAnswer: defaultAnswer answerHeight: answerHeight
	"Create a multi-line instance of me whose question is queryString with
	the given initial answer. Invoke it centered at the given point, and
	answer the string the user accepts.  Answer nil if the user cancels.  An
	empty string returned means that the ussr cleared the editing area and
	then hit 'accept'.  Because multiple lines are invited, we ask that the user
	use the ENTER key, or (in morphic anyway) hit the 'accept' button, to 
	submit; that way, the return key can be typed to move to the next line."
	^FillInTheBlank multiLineRequest: queryString centerAt: aPoint initialAnswer: defaultAnswer answerHeight: answerHeight! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/11/2005 15:43'!
requestPassword: queryString
	"Create an instance of me whose question is queryString. Invoke it centered
	at the cursor, and answer the string the user accepts. Answer the empty 
	string if the user cancels."
	^FillInTheBlank requestPassword: queryString! !

!MVCUIManager methodsFor: 'ui requests' stamp: 'ar 2/11/2005 15:43'!
request: queryString initialAnswer: defaultAnswer 
	"Create an instance of me whose question is queryString with the given 
	initial answer. Invoke it centered at the given point, and answer the 
	string the user accepts. Answer the empty string if the user cancels."
	^FillInTheBlank request: queryString initialAnswer: defaultAnswer ! !
